/*
** $Id: daemonise.c,v 1.1 2010/08/01 22:42:50 A04072 Exp A04072 $
**
** Execute another program as a daemon.
**
** Usage: daemonise [-c] [-f] command args ...
**
** where -c: says change directory to '/'
**       -f: says close file descriptors 0, 1, and 2
**
** If you don't use the -c option, the current working directory
** should be established suitably for the daemon.
**
** If you don't use the -f option, stdin, stdout, and stderr must
** be redirected (usually by the shell) away from a tty -- usually
** either to /dev/null or a regular file.
**
** Daemon(3) calls setsid(2) to become the session leader of the
** new session, and the process group leader of a new process group
** which has no controlling terminal.  By abandoning our original
** controlling terminal, session, and process group we become immune
** to signals that we might have otherwise received.
**
** The "command" should not open any tty, as this will cause it to 
** acquire the first tty opened as a controlling terminal (this will
** re-expose us to process group signals that we don't want).
**
** See setsid(2) and daemon(3) for further information.
**
** Phil Chadwick.  2010-07-29.
**
*/

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

static void usage(void);
static char *ProgName = (char *) 0;
extern int optind;

int
main(int argc, char *argv[])
{
    int c;
    int nochdir = 1;
    int noclose = 1;

    ProgName = strrchr(argv[0], '/');
    if ( ProgName == (char *) 0 )
	ProgName = argv[0];
    else
	ProgName++;

    while ((c = getopt(argc, argv, "cf")) != -1)
    {
	switch (c)
	{
	    case 'c':
		nochdir = 0;
		break;
	    case 'f':
		noclose = 0;
		break;
	    default:
		usage();
	}
    }
    argv += optind;
    argc -= optind;

    if ( argc == 0 )
	usage();

    if ( daemon(nochdir, noclose) != -1 )
	execvp(argv[0], argv);

    perror(ProgName);

    exit(1);
}

static void
usage(void)
{
    (void) fprintf(stderr, "usage: %s [-c] [-f] command args ...\n", ProgName);
    exit(1);
}

