/*
** basic daemon.c for System V derivitives
** (which require two forks).
** Linux and BSD have daemon.c in libc.
** 
** Phil Chadwick
*/

#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

extern int errno;
int ighup_fork(void);


/* 
** Detatch from the controlling terminal, and become group leader
** of a new process group with a new session ID.  Optionally change
** working directory to root, and close all file descriptors.
*/
int
daemon(int nochdir, int noclose)
{
    if ( ighup_fork() < 0 )
        return(-1);

    if ( setsid() < 0 )
        return(-1);

    if ( ighup_fork() < 0 )
        return(-1);

    if ( ! nochdir )
	(void) chdir("/"); 

    if ( ! noclose )
    {
	int n;
	int maxfd = getdtablesize();
	for (n = 0; n < maxfd; n++)
        {
	    (void) close(n);
	}
	(void) open("/dev/null", O_RDONLY);
	(void) open("/dev/null", O_RDWR);
	(void) open("/dev/null", O_RDWR);
    }

    return(0);
}

/* 
** fork, parent exits, child ignores SIGHUP
** which may be generated by parent's exit.
** returns 0 for success, -1 if fork failed.
*/
int
ighup_fork(void)
{

    struct  sigaction psa, sa;
    int     sa_status;
    int	    perrno;

    (void) sigemptyset(&sa.sa_mask);
    sa.sa_handler = SIG_IGN;
    sa.sa_flags = 0;
    sa_status = sigaction(SIGHUP, &sa, &psa);

    int pid = fork();

    if ( pid > 0 )
        _exit(0);	/* parent */

    if ( sa_status != -1 )
    {
	perrno = errno;
	(void) sigaction(SIGHUP, &psa, (struct sigaction *) 0);
	errno = perrno;
    }

    return(pid);
}
